#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      http_client.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    tangmi(tangmi360@gmail.com)
    @date:      June 11, 2018
    @desc:      Basic http client functions
"""

from aiohttp import ClientSession


class HttpClient(ClientSession):
    def __init__(self, *args, **kwargs):
        super(HttpClient, self).__init__(*args, **kwargs)



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      mongo_helper.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    tangmi(tangmi360@gmail.com)
    @date:      September 04, 2018
    @desc:      Mongo Helper
"""

from bson.objectid import ObjectId


class MongoHelper(object):

    def __init__(self, *args, **kwargs):
        self.key_map = {}
        self._coll = args[0]

    def mapping(self, item, result=None):
        if result == None:
            result = {}
        for k, v in item.items():
            k = self.key_map.get(k, k)
            if isinstance(v, dict):
                result[k] = {}
                mapping(v, result[k])
            else:
                result[k] = v
        return result

    def __getattr__(self, attr):
        return getattr(self._coll, attr)

    async def find_by_id(self, _id):
        doc = await self._coll.find_one({'_id': ObjectId(_id)})
        return doc



#!/usr/bin/python
# -*- coding:utf-8 -*-

from functools import wraps


def singleton(cls):
    _instances = {}
    @wraps(cls)
    def _singleton(*args, **kwargs):
        if cls not in _instances:
            _instances[cls] = cls(*args, **kwargs)
        return _instances[cls]
    return _singleton



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      sql_param_collections.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    zhouqinmin(zqm175899960@163.com)
    @date:      January 23, 2019
    @desc:      SqlParamCollections
"""

# normal_param_list : normal sql param, eg: A = 1
# normal_linker_op_list : link op symbol, link normal sql param, eg: and or
# group_by_param_list : group by param, eg: A
# having_param_list : having parmam, eg: having sum(A) > 100
# order_by_param_list : order by params eg: order by A asc
# limit_param : limit param eg: limit 1, 10
import functools
from collections import namedtuple


NormalSqlParam = namedtuple('NormalSqlParam', ['key', 'operator', 'value', 'is_normal']) #A = 10#A in (1, 2, 3)#A between 1 and 2
LimitSqlParam = namedtuple('LimitSqlParam', ['offset', 'rows']) #limit 0, 10
GroupBySqlParam = namedtuple('GroupBySqlParams', ['key']) #group by A
OrderBySqlParam = namedtuple('OrderBySqlParam', ['key', 'order_dsc']) #order by A asc
HavingSqlParam = namedtuple('HavingSqlParam', ['key', 'operator', 'value']) 
SelectClauseParam = namedtuple('SelectClauseParam', ['sql', 'params'])

class SqlParamCollections(object):
    def __init__(self, 
            normal_param_list=[], 
            normal_linker_op_list = [], 
            group_by_param_list=[],
            having_param_list = [], 
            having_linker_op = [],
            order_by_param_list=[],
            limit_param=None
            ):
        self.__data = dict()
        self.__data['normal_param'] = list()
        self.__data['linker_op_list'] = list()
        self.__data['limit'] = list()
        self.__data['order_by'] = list()
        self.__data['group_by'] = list()
        self.__data['having'] = list()
        self.__data['having_linker_op_list'] = list()
        if normal_param_list and len(normal_param_list) != 0:
            self.extend_normal_param(normal_param_list)
        if order_by_param_list and len(order_by_param_list) != 0:
            self.__data['order_by'].extend(order_by_param_list)
        if group_by_param_list and len(group_by_param_list) != 0:
            self.__data['group_by'].extend(group_by_param_list)
        if normal_linker_op_list and len(normal_linker_op_list) != 0:
            self.__data['linker_op_list'].extend(normal_linker_op_list)
        if having_param_list and len(having_param_list) != 0:
            self.__data['having'].extend(having_param_list)
        if having_linker_op and len(having_linker_op) != 0:
            self.__data['having_linker_op_list'].extend(having_linker_op)
        if limit_param is not None:
            self.__data['limit'].append(limit_param)

        self.make_functions = {
            'normal_param':functools.partial(SqlParamCollections.make_where_sql_clause, params_data=self.__data['normal_param'], linker_op_data=self.__data['linker_op_list']),
            'group_by':functools.partial(SqlParamCollections.make_groupby_sql_clause, data=self.__data['group_by']),
            'order_by':functools.partial(SqlParamCollections.make_orderby_sql_clause, data=self.__data['order_by']),
            'limit':functools.partial(SqlParamCollections.make_limit_sql_clause, data=self.__data['limit']),
            'having':functools.partial(SqlParamCollections.make_having_sql_clause, params_data=self.__data['having'], linker_op_data=self.__data['having_linker_op_list'])
        }

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, value):
        self.__data = value

    def add_normal_param(self, normal_param):
        self.__data['normal_param'].append(normal_param)
        return len(self.__data)

    def extend_normal_param(self, normal_param_list):
        self.__data['normal_param'].extend(normal_param_list)

    def add_group_by_param(self, group_by_param):
        self.__data['group_by'].append(group_by_param)

    def add_order_by_param(self, order_by_param):
        self.__data['order_by'].append(order_by_param)

    def add_having_param(self, having_param):
        self.__data['having'].append(having_param)

    def extend_having_param(self, having_param_list):
        self.__data['having'].extend(having_param_list)

    def extend_having_linker_op(self, having_linker_op_list):
        self.__data['having_linker_op_list'].extend(having_linker_op_list)

    def add_having_linker_op(self, having_linker_op):
        self.__data['having_linker_op_list'].append(having_linker_op)

    def set_limit_param(self, limit_param):
        self.__data['limit'] = limit_param

    def add_linker_op(self, linker_op):
        self.__data['linker_op_list'].append(linker_op)

    def extend_linker_op(self, linker_op_list):
        self.__data['linker_op_list'].extend(linker_op_list)

    @staticmethod
    def make_where_sql_clause(params_data, linker_op_data):
        str_noraml_param_list = list()
        param_table_list = list()
        for p in params_data:
            op = p[1].lower()
            if isinstance(p[2], SelectClauseParam):
                tmp_list = []
                tmp_list.append('%s %s ' % (p[0], op))
                tmp_list.append('(')
                tmp_list.append(p[2][0])
                tmp_list.append(')')
                str_noraml_param_list.append(''.join(tmp_list))
                param_table_list.extend(p[2][1])
                continue
            if not p[3]:
                str_noraml_param_list.append('%s %s %s' % (p[0], op, p[2]))
                continue
            in_flags = False if op.find('in') == -1 else True
            between_flags = False if op.find('between') == -1 else True
            if not in_flags and not between_flags:
                str_noraml_param_list.append('%s %s %%s' % (p[0], op))
                param_table_list.append(p[2])
                continue
            else:
                if isinstance(p[2], list) or isinstance(p[2], tuple):
                    param_list = list(p[2])
                else:
                    param_list = p[2].split(',')
                if between_flags:
                    if len(param_list) == 1:
                        param_list = param_list * 2
                    param_list = param_list[:2]
                    str_noraml_param_list.append('%s %s %s' % (p[0], op, ' and '.join(['%s'] * len(param_list))))
                else:
                    str_noraml_param_list.append('%s %s (%s)' % (p[0], op, ', '.join(['%s'] * len(param_list))))
                param_table_list.extend(param_list)
        normal_param_list_count = len(str_noraml_param_list)
        copy_linker_op_data = [op for op in linker_op_data]
        while len(copy_linker_op_data) < normal_param_list_count:
            copy_linker_op_data.append('and')

        result_list = list()
        for i in range(len(str_noraml_param_list)):
            result_list.append(str_noraml_param_list[i])
            result_list.append(copy_linker_op_data[i])
        result_list.pop()

        return (' '.join(result_list), param_table_list)

    @staticmethod
    def make_having_sql_clause(params_data, linker_op_data):
        result_list = []
        result_list.append(' having ')
        (sql, params) = SqlParamCollections.make_where_sql_clause(params_data, linker_op_data)
        result_list.append(sql)
        return (''.join(result_list), params)

    @staticmethod
    def make_orderby_sql_clause(data):
        result_list = []
        result_list.append(' order by')
        orderby_clause_list = []
        for p in data:
            if isinstance(p, tuple) and len(p) == 2:
                orderby_clause_list.append('%s %s' % (p[0], p[1]))
            else:
                orderby_clause_list.append(p)
        result_list.append(', '.join(orderby_clause_list))
        return (' '.join(result_list), [])

    @staticmethod
    def make_groupby_sql_clause(data):
        result_list = []
        result_list.append(' group by ')
        result_list.append(', '.join([p for p in data]))
        return (''.join(result_list), [])

    @staticmethod
    def make_limit_sql_clause(data):
        if isinstance(data[0], tuple):
            copy_limit_param = [op for op in list(data[0])]
        else:
            copy_limit_param = [op for op in data]
        if len(copy_limit_param) != 2:
            copy_limit_param.insert(0, 0)
        return (' limit %s, %s' % (copy_limit_param[0], copy_limit_param[1]), [])

    def can_add_where(self):
        return len(self.__data['normal_param'])

    def get_where_sql(self):
        result_list = []
        result_params = []
        sql_key_word = ['normal_param', 'group_by', 'having', 'order_by', 'limit']
        for k in sql_key_word:
            if len(self.__data[k]) != 0:
                (sql, params) = self.make_functions[k]()
                result_list.append(sql)
                result_params.extend(params)
        return (''.join(result_list), result_params)

def make_example():
    sql_params = SqlParamCollections(
        normal_param_list=[
            ('A','=', 1, True), 
            ('B', '<', 2, True),
            ('time','between',[1, 2], True),
            ('C','not in',[1, 3, 4], True)
        ], 
        group_by_param_list=[
            'B'
        ], 
        having_param_list=[
            ('A', '>', 100, True), 
            ('B', '!=', 2, True)
        ], 
        having_linker_op=[
            'or'
        ], 
        order_by_param_list=[
            ('B', 'asc'), 
            ('A', 'desc')
        ], 
        normal_linker_op_list=[
            'or', 
            'or'
        ],
        limit_param=(20))
    print(sql_params.get_where_sql())
    sql_params = SqlParamCollections(
        normal_param_list=[
            ('A','=', 1, True), 
            ('B', '<', 2, True)]
        )
    print(sql_params.get_where_sql())

def main():
    make_example()

if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding:utf-8 -*-

from .mysql_module import MySQLModule
from .mysql_module import MySQLModule as MySQL
from .mysql_controller import MysqlController
from .mysql_batch_controller import MysqlBatchController
from .sql_clause_assemble import SqlClauseAssemble
from .sql_param_collections import SqlParamCollections



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      mongo_client.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    tangmi(tangmi360@gmail.com)
    @date:      June 4, 2018
    @desc:      Mongodb storage access class
"""

from motor.motor_asyncio import AsyncIOMotorClient


class MongoClient(AsyncIOMotorClient):

    def __init__(self, *args, **kwargs):
        if kwargs.get('authMechanism') == None:
            kwargs['authMechanism'] = 'SCRAM-SHA-1'
        super(MongoClient, self).__init__(*args, **kwargs)



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      mysql_controller.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    zhouqinmin(zqm175899960@163.com)
    @date:      January 23, 2019
    @desc:      MysqlController
"""

#fit for normal add delete update select
import logging
import datetime
from tomato.store.mysql.mysql_model import MySqlModel
from tomato.store.mysql.sql_clause_assemble import BatchOperator
from tomato.store.mysql.sql_clause_assemble import SqlClauseAssemble


class MysqlController(object):
    __mysql_client = None
    def __init__(self):
        self.wanted_words = list()
        self.table_name = ''
        self.batch_clause_functions = dict()
        self.batch_clause_functions = {
            BatchOperator.INSERT: self._get_insert_clause,
            BatchOperator.DELETE: self._get_delete_clause,
            BatchOperator.UPDATE: self._get_update_clause,
            BatchOperator.SELECT: self._get_query_clause
        }

    @classmethod
    def set_mysql_client(cls, mysql_client):
        cls.__mysql_client = mysql_client

    async def parse_row(self, row):
        model = MySqlModel()
        for k, v in row.items():
            if isinstance(v, datetime.datetime):
                v = v.strftime("%Y-%m-%d %H:%M:%S")
            model.update({k: v})
        return model

    async def delete(self, where_params=None):
        (sql_clause, params) = self.execute_sql_clause(BatchOperator.DELETE, None, where_params)
        logging.info(((sql_clause, params)))
        result = await self.__mysql_client.execute(sql_clause, tuple(params))
        return result

    async def insert(self, mysql_model, where_params=None):
        (sql_clause, params) = self.execute_sql_clause(BatchOperator.INSERT, mysql_model, where_params)
        logging.info(((sql_clause, params)))
        result = await self.__mysql_client.execute(sql_clause, tuple(params))
        return result

    async def query(self, wanted_words=[], where_params=None):
        data_list = []
        (sql_clause, params) = self.execute_sql_clause(BatchOperator.SELECT, wanted_words, where_params)
        logging.info(((sql_clause, params)))
        result = await self.__mysql_client.get_all(sql_clause, tuple(params))
        if result is not None and len(result) > 0:
            for row in result:
                data = await self.parse_row(row)
                data_list.append(data)
        return data_list

    async def update(self, mysql_model, where_params=None):
        (sql_clause, params) = self.execute_sql_clause(BatchOperator.UPDATE, mysql_model, where_params)
        logging.info((sql_clause, params))
        result = await self.__mysql_client.execute(sql_clause, params)
        return result

    async def query_count(self, where_params=None):
        count = 0
        wanted_words = ['count(0)']
        result = await self.query(wanted_words, where_params)
        if result is not None and len(result) > 0:
            count = result[0]['count(0)']
        return count

    async def query_maxid(self, where_params=None):
        max_id = 0
        wanted_words = ['max(id)']
        result = await self.query(wanted_words, where_params)
        if result is not None and len(result) > 0:
            max_id = result[0]['max(id)']
        return max_id

    def execute_sql_clause(self, BatchOperator, mysql_model, where_params):
        return self.batch_clause_functions[BatchOperator](mysql_model, where_params)

    def _get_insert_clause(self, mysql_model, where_params):
        sql_assemble = SqlClauseAssemble()
        sql_assemble.table_name = self.table_name
        sql_assemble.insert_words = mysql_model.items()
        sql_assemble.where_params = where_params
        return sql_assemble.get_insert_clause()

    def _get_query_clause(self, wanted_words, where_params):
        sql_assemble = SqlClauseAssemble()
        sql_assemble.table_name = self.table_name
        sql_assemble.wanted_words = wanted_words \
            if wanted_words and len(wanted_words) != 0 else self.wanted_words
        sql_assemble.where_params = where_params
        return sql_assemble.get_query_clause()

    def _get_update_clause(self, mysql_model, where_params):
        sql_assemble = SqlClauseAssemble()
        sql_assemble.table_name = self.table_name
        sql_assemble.update_words = mysql_model.items()
        sql_assemble.where_params = where_params
        return sql_assemble.get_update_clause()

    def _get_delete_clause(self, mysql_model, where_params):
        sql_assemble = SqlClauseAssemble()
        sql_assemble.table_name = self.table_name
        sql_assemble.where_params = where_params
        return sql_assemble.get_delete_clause()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
import traceback
import logging
import logging.handlers

from tomato.util import loadconf
from tomato.util.singleton import singleton


@singleton
class ColoredFormatter(logging.Formatter):
    '''A colorful formatter.'''
 
    def __init__(self, fmt = None, datefmt = None):
        logging.Formatter.__init__(self, fmt, datefmt)
 
    def format(self, record):
        # Color escape string
        COLOR_RED='\033[1;31m'
        COLOR_GREEN='\033[1;32m'
        COLOR_YELLOW='\033[1;33m'
        COLOR_BLUE='\033[1;34m'
        COLOR_PURPLE='\033[1;35m'
        COLOR_CYAN='\033[1;36m'
        COLOR_GRAY='\033[1;37m'
        COLOR_WHITE='\033[1;38m'
        COLOR_RESET='\033[1;0m'
        # Define log color
        LOG_COLORS = {
            'DEBUG': '%s',
            'INFO': COLOR_GREEN + '%s' + COLOR_RESET,
            'WARNING': COLOR_YELLOW + '%s' + COLOR_RESET,
            'ERROR': COLOR_RED + '%s' + COLOR_RESET,
            'CRITICAL': COLOR_RED + '%s' + COLOR_RESET,
            'EXCEPTION': COLOR_RED + '%s' + COLOR_RESET,
        }
        level_name = record.levelname
        msg = logging.Formatter.format(self, record)
        return LOG_COLORS.get(level_name, '%s') % msg

@singleton
class Log(object):

    def __init__(self, conf_file=None,
             filename=None,
             mode='a',
             cmdlevel='INFO',
             filelevel='INFO',
             cmdfmt = '[%(asctime)s] %(levelname)s %(process)d %(filename)s:%(lineno)d %(message)s',
             filefmt = '[%(asctime)s] %(levelname)s %(process)d %(filename)s:%(lineno)d %(message)s',
             cmddatefmt = '%Y-%m-%d %H:%M:%S',
             filedatefmt = '%Y-%m-%d %H:%M:%S',
             backup_count = 0, limit = 20480, when = None, colorful = False):

        logging.getLogger().level = logging.NOTSET
        # logging.debug('logger init...')

        self.filename = filename
        if conf_file != None:
            conf = loadconf(conf_file, raw=True)
            conf_items = dict(conf.items('settings'))
            if 'filename' in conf_items:
                self.filename = conf_items['filename']

        if self.filename is None:
            self.filename = getattr(sys.modules['__main__'], '__file__', 'log.py')
            self.filename = os.path.basename(self.filename.replace('.py', '.log'))

        if not os.path.exists(os.path.abspath(os.path.dirname(self.filename))):
            os.makedirs(os.path.abspath(os.path.dirname(self.filename)))
        self.mode = mode
        self.cmdlevel = cmdlevel
        self.filelevel = filelevel
        if isinstance(self.cmdlevel, str):
            self.cmdlevel = getattr(logging, self.cmdlevel.upper(), logging.DEBUG)
        if isinstance(self.filelevel, str):
            self.filelevel = getattr(logging, self.filelevel.upper(), logging.DEBUG)
        self.filefmt = filefmt
        self.cmdfmt = cmdfmt
        self.filedatefmt = filedatefmt
        self.cmddatefmt = cmddatefmt
        self.backup_count = backup_count
        self.limit = limit
        self.when = when
        self.colorful = colorful
        self.logger = None
        self.streamhandler = None
        self.filehandler = None
        self.set_logger(cmdlevel=self.cmdlevel)

        if conf_file != None:
            self.set_logger(mode=conf_items['mode'],
                            cmdlevel=conf_items['cmdlevel'],
                            filelevel=conf_items['filelevel'],
                            cmdfmt=conf_items['cmdfmt'],
                            filefmt=conf_items['filefmt'],
                            cmddatefmt=conf_items['cmddatefmt'],
                            filedatefmt=conf_items['filedatefmt'],
                            backup_count=int(conf_items['backup_count']),
                            when=conf_items['when'],
                            colorful=True)
        self.debug('logger initialization completed')

    def init_logger(self):
        '''Reload the logger.'''
        if self.logger is None:
            self.logger = logging.getLogger()
        else:
            logging.shutdown()

        self.logger.handlers = []
        self.streamhandler = None
        self.filehandler = None
        self.logger.setLevel(logging.DEBUG)
 
    def add_streamhandler(self):
        '''Add a stream handler to the logger.'''
        self.streamhandler = logging.StreamHandler()
        self.streamhandler.setLevel(self.cmdlevel)
        if self.colorful:
            formatter = ColoredFormatter(self.cmdfmt, self.cmddatefmt)
        else:
            formatter = logging.Formatter(self.cmdfmt, self.cmddatefmt,)
        self.streamhandler.setFormatter(formatter)
        self.logger.addHandler(self.streamhandler)

    def add_filehandler(self):
        '''Add a file handler to the logger.'''
        # Choose the filehandler based on the passed arguments
        if self.backup_count == 0: # Use FileHandler
            self.filehandler = logging.FileHandler(self.filename, self.mode)
        elif self.when is None:  # Use RotatingFileHandler
            self.filehandler = logging.handlers.RotatingFileHandler(self.filename,
                    self.mode, self.limit, self.backup_count)
        else: # Use TimedRotatingFileHandler
            self.filehandler = logging.handlers.TimedRotatingFileHandler(self.filename,
                    self.when, 1, self.backup_count)
        self.filehandler.setLevel(self.filelevel)
        formatter = logging.Formatter(self.filefmt, self.filedatefmt)
        self.filehandler.setFormatter(formatter)
        self.logger.addHandler(self.filehandler)
 
    def set_logger(self, **kwargs):
        '''Configure the logger.'''
        keys = ['mode','cmdlevel','filelevel','filefmt','cmdfmt',\
                'filedatefmt','cmddatefmt','backup_count','limit',\
                'when','colorful']
        for (key, value) in kwargs.items():
            if not (key in keys):
                return False
            setattr(self, key, value)
        if isinstance(self.cmdlevel, str):
            self.cmdlevel = getattr(logging, self.cmdlevel.upper(), logging.DEBUG)
        if isinstance(self.filelevel, str):
            self.filelevel = getattr(logging, self.filelevel.upper(), logging.DEBUG)
        if not "cmdfmt" in kwargs:
            self.filefmt='[%(asctime)s] %(filename)s line:%(lineno)d %(levelname)-8s%(message)s'
            self.filedatefmt = '%Y-%m-%d %H:%M:%S'
            self.cmdfmt='[%(asctime)s] %(filename)s line:%(lineno)d %(levelname)-8s%(message)s'
            self.cmddatefmt = '%Y-%m-%d %H:%M:%S'

        self.init_logger()
        self.add_streamhandler()
        self.add_filehandler()
        # Import the common log functions for convenient
        self.import_log_funcs()
        return True

    def import_log_funcs(self):
        '''Import the common log functions from the logger to the class'''
        log_funcs = ['debug', 'info', 'warning', 'error', 'critical',
                     'exception']
        for func_name in log_funcs:
            func = getattr(self.logger, func_name)
            setattr(self, func_name, func)

    def trace(self):
        info = sys.exc_info()
        for file, lineno, function, text in traceback.extract_tb(info[2]):
            self.error('%s line:%s in %s:%s' % (file, lineno, function, text))
        self.error('%s: %s' % info[:2])



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      mysql_client.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    tangmi(tangmi360@gmail.com), zhouqinmin(zqm175899960@163.com)
    @date:      June 11, 2018
    @desc:      MySQL storage access class
"""

import logging
import aiomysql
from aiomysql.cursors import DictCursor
from tomato.util.appmodule import AppModule
from tomato.store.mysql.mysql_batch_controller import MysqlBatchController

class MySQLModule(AppModule):

    def __init__(self, *args, **kwargs):
        super(MySQLModule, self).__init__(*args, **kwargs)
        setting = kwargs.get('setting', None)
        if setting: kwargs.update(setting)
        self._db = kwargs.get('db')
        self._host = kwargs.get('host', 'localhost')
        self._port = int(kwargs.get('port', 3306))
        self._user = kwargs.get('user')
        self._passwd = kwargs.get('passwd')
        self._minsize = int(kwargs.get('minsize', 5))
        self._maxsize = int(kwargs.get('maxsize', 5))
        self._charset = kwargs.get('charset', 'utf8')
        self._pool = None

    def setup(self):
        pass

    async def run(self):
        self._pool = await aiomysql.create_pool(host=self._host,
                                                port=self._port,
                                                user=self._user,
                                                password=self._passwd,
                                                db=self._db,
                                                minsize=self._minsize,
                                                maxsize=self._maxsize,
                                                charset=self._charset,
                                                cursorclass=DictCursor)
        async with self._pool.acquire() as conn:
            async with conn.cursor() as cur:
                await cur.execute("SELECT 1;")
                (r,) = await cur.fetchone()
                assert r == '1'
                logging.info('mysql connection success: mysql://%s:%s'
                             ';charset=%s, db[%s] '
                             'minsize[%s] maxsize[%s]',
                             self._host, self._port, self._charset,
                             self._db, self._minsize, self._maxsize)
        MysqlBatchController(self)

    async def destroy(self):
        if self._pool:
            self._pool.close()
            await self._pool.wait_closed()

    async def execute(self, sql, args):
        ret = False
        async with self._pool.acquire() as conn:
            try:
                async with conn.cursor() as cur:
                    await cur.execute(sql, args)
                    affected = cur.rowcount
                    if affected > 0:
                        ret = True
            except Exception as e:
                logging.exception(e)
                logging.exception(sql)
        return ret

    async def execute_batch(self, params):
        ret = False
        async with self._pool.acquire() as conn:
            try:
                await conn.begin()
                async with conn.cursor(aiomysql.DictCursor) as cur:
                    for p in params:
                        await cur.execute(p[0], p[1])
                    await conn.commit()
                    ret = True
            except Exception as e:
                await conn.rollback()
                logging.exception(e)
        return ret

    async def get_all(self, sql, args):
        results = None
        async with self._pool.acquire() as conn:
            try:
                async with conn.cursor() as cur:
                    await cur.execute(sql, args)
                    results = await cur.fetchall()
            except Exception as e:
                logging.exception(e)
                logging.exception(sql)
        return results

    async def get_one(self, sql, args):
        results = None
        async with self._pool.acquire() as conn:
            try:
                async with conn.cursor() as cur:
                    await cur.execute(sql, args)
                    results = await cur.fetchone()
            except Exception as e:
                logging.exception(e)
                logging.exception(sql)
        return results



#!/usr/bin/python
# -*- coding:utf-8 -*-

from functools import wraps
from inspect import signature


def typeassert(*ty_args, **ty_kwargs):
    def decorate(func):
        # If in optimized mode, disable type checking
        if not __debug__:
            return func

        # Map function argument names to supplied types
        sig = signature(func)
        bound_types = sig.bind_partial(*ty_args, **ty_kwargs).arguments

        @wraps(func)
        def wrapper(*args, **kwargs):
            bound_values = sig.bind(*args, **kwargs)
            # Enforce type assertions across supplied arguments
            for name, value in bound_values.arguments.items():
                if name in bound_types:
                    if not isinstance(value, bound_types[name]):
                        raise TypeError(
                            'Argument {} must be {}'.format(name, bound_types[name])
                            )
            return func(*args, **kwargs)
        return wrapper
    return decorate



#!/usr/bin/python
# -*- coding:utf-8 -*-

import unittest
from tomato.store import MongoClient


class TestMongoClient(unittest.TestCase):

    def setUp(self):
        print('setUp...')

    def tearDown(self):
        print('tearDown...')

    def test_open(self):
        pass

    def test_insert(self):
        pass

    def test_update(self):
        pass

    def test_select(self):
        pass

    def test_close(self):
        pass



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      http_server.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    tangmi(tangmi360@gmail.com)
    @date:      June 11, 2018
    @desc:      Basic http server functions and router rules encapsulation
"""

import json
import logging
from aiohttp import web
from tomato.transport.http import index_html
from tomato.util.appmodule import AppModule


class HttpServerModule(AppModule):

    def __init__(self, *args, **kwargs):
        super(HttpServerModule, self).__init__(*args, **kwargs)
        setting = kwargs.get('setting', None)
        if setting: kwargs.update(setting)
        self._host = args[0] if len(args) > 0 else kwargs.get('host', 'localhost')
        self._port = int(args[1] if len(args) > 1 else kwargs.get('port', '1024'))
        self._services = kwargs.get('services', [])

        self._app = web.Application()
        # self._app = web.Application(middlewares=[self.base_middleware,])
        self._app.router.add_route('get', '/', self.default_handler)
        for service in self._services:
            self._app.router.add_routes(service.routes)

    def setup(self):
        pass

    async def run(self):
        self._runner = web.AppRunner(self._app)
        await self._runner.setup()
        site = web.TCPSite(self._runner, self._host, self._port)
        await site.start()
        logging.info('serving on [%s]', site.name)

    async def destroy(self):
        await self._runner.cleanup()
        await self._app.shutdown()
        await self._app.cleanup()

    async def close(self):
        await self.destroy()

    def default_handler(self, request):
        req_params = request.query
        response = web.Response(body=index_html.text.encode('utf-8'))
        response.headers['Content-Language'] = 'en'
        response.headers['Content-Type'] = 'en'
        return response

    @web.middleware
    async def base_middleware(self, request, handler):
        """http简单解包与封包
           1.qs信息从request.query属性中直接获取（MultiDictProxy类型）
           2.http body内容如果请求头为content_type=application/json,
             则通过json标准库进行解析，并覆盖request.body，
             否则直接把body内容抛到业务层自行处理（默认是byte类型）
           3.对于response的处理，目前简单判断为dict则直接转成json格式str发给调用端
             否则直接按业务层返回值返回给调用端
        """
        request.body = None
        if request.body_exists and request.can_read_body:
            request.body = await request.content.read()
            if request.content_type == 'application/json':
                try:
                    request.body = json.loads(request.body)
                except Exception as e:
                    logging.warning('json format warning, errmsg[%s]', str(e))
                    return web.json_response({'ret': 0, 'msg': 'params error'})
        logging.warning('type[%s]', type(handler))
        response = await handler(request)
        if isinstance(response, dict):
            return web.json_response(response)
        else:
            return web.Response(body=response)

        response = await handler(request)
        return response



#!/usr/bin/python
# -*- coding:utf-8 -*-


import json
import ujson
import logging


logging.basicConfig(level=logging.INFO)

def patch_json():
    json.__name__ = 'ujson'
    json.dumps = ujson.dumps
    json.loads = ujson.loads



#!/usr/bin/python
# -*- coding:utf-8 -*-

from aiohttp.web import RouteTableDef


class Routes(RouteTableDef):
    def __init__(self, *args, **kwargs):
        RouteTableDef.__init__(self, *args, **kwargs)



#!/usr/bin/python
# -*- coding:utf-8 -*-


from .mysql import MySQLModule
from .mysql import MySQLModule as MySQL
from .mysql import MysqlController
from .mysql import MysqlBatchController
from .mysql import SqlClauseAssemble
from .mysql import SqlParamCollections

from .leveldb import LevelDB
from .redis_client import RedisClient
from .mongo import MongoClient
from .mongo import MongoHelper


#!/usr/bin/python
# -*- coding:utf-8 -*-

import sys
import signal
import asyncio
import logging
import functools
from tomato.util import singleton
from tomato.util import typeassert
from tomato.util.appmodule import AppModule


@singleton
class Application(object):

    def __init__(self, *args, **kwargs):
        self.__running = False
        self.__modules = {}
        self.__module_names = []

    async def __func_call(self, func):
        iscoroutine = asyncio.iscoroutinefunction(func)
        if iscoroutine:
            await func()
        else:
            func()

    async def __run(self):
        self.__running = True
        try:
            for name in self.__module_names:
                module = self.__modules[name]
                await self.__func_call(module.setup)
                await self.__func_call(module.run)
        except Exception as e:
            logging.exception(e)
            try:
                await self.__destroy()
            finally:
                sys.exit(1)

    async def __destroy(self):
        for name in self.__module_names:
            module = self.__modules[name]
            await self.__func_call(module.destroy)
        self.__running = False
        logging.info('server has stopped')

    def __add_signal(self, future, loop):
        def shutdown(signame, future, loop):
            logging.warning('got signal %s: server is about to stop', signame)
            future.cancel() # TODO  if the outer Future is cancelled, all children (that have not completed yet) are also cancelled.
            loop.stop()
        loop.add_signal_handler(signal.SIGINT, functools.partial(shutdown, 'SIGINT', future, loop))
        loop.add_signal_handler(signal.SIGHUP, functools.partial(shutdown, 'SIGHUP', future, loop))
        loop.add_signal_handler(signal.SIGTERM, functools.partial(shutdown, 'SIGTERM', future, loop))

    def run(self):
        if self.__running:
            logging.warning('application already running')
            return

        loop = asyncio.get_event_loop()
        future = asyncio.gather(self.__run())
        self.__add_signal(future, loop)
        loop.run_until_complete(future)

    def stop(self):
        if self.__running:
            loop = asyncio.get_event_loop()
            loop.run_until_complete(self.__destroy())

    def run_forever(self):
        if self.__running:
            logging.warning('application already running')
            return

        loop = asyncio.get_event_loop()
        future = asyncio.gather(self.__run())
        self.__add_signal(future, loop)
        loop.run_until_complete(future)
        loop.run_forever()
        loop.run_until_complete(self.__destroy())

    # @typeassert(str, AppModule)
    def add_module(self, name, module):
        if self.__running:
            logging.error('application running, cannot add module[%s]', name)
            return

        if name in self.__modules:
            logging.warning('[%s] module with the same name already exists', name)
            return

        self.__module_names.append(name)
        self.__modules[name] = module

    def get_module(self, name):
        return self.__modules.get(name, None)

    def print_modules(self):
        logging.info(self.__modules)

    def __getitem__(self, name):
        return self.get_module(name)

    def __setitem__(self, name, module):
        self.add_module(name, module)



#!/usr/bin/python
# -*- coding:utf-8 -*-


text = '''<html>
<head>
<title>Welcome to Tomato!</title>
<style>
    body {
        width: 35em;
        margin: 0 auto;
        font-family: Tahoma, Verdana, Arial, sans-serif;
    }
</style>
</head>
<body>
<h1>Welcome to Tomato!</h1>
<p>If you see this page, the tomato server is successfully installed and working.</p>
<p>For online documentation and support please refer to 
<a href="https://github.com/tangmi001/tomatolib/" target="_blank">tomatolib</a>.<br />

<p><em>Thank you for use tomatolib.</em></p>
</body>
</html>
'''



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      mysql_model.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    zhouqinmin(zqm175899960@163.com)
    @date:      January 23, 2019
    @desc:      MySqlModel
"""

from decimal import Decimal
from datetime import datetime


class MySqlModel(dict):
    def __init__(self, **kwargs):
        super(MySqlModel, self).__init__(kwargs)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError(r"'Dict' object has no attribute '%s'" % key)

    def __setattr__(self, key, value):
        self[key] = value

    def to_display_dict(self):
        result = dict()
        for k, v in self.items():
            if isinstance(v, datetime):
                v = v.strftime("%Y-%m-%d %H:%M:%S")
            elif isinstance(v, Decimal):
                v = str(v)
            result.update({k: v})
        return result

def test():
    model = MySqlModel()
    model['name'] = 'test'
    model.age = 32
    print(model)

def main():
    test()

if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding:utf-8 -*-

from .http_client import HttpClient
from .http_server_module import HttpServerModule
from .http_server_module import HttpServerModule as HttpServer
from .routes import Routes

routes = Routes()



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      leveldb.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    tangmi(tangmi360@gmail.com)
    @date:      September 04, 2018
    @desc:      LevelDB
"""

import os
import leveldb

class LevelDB(object):
    def __init__(self, *args, **kwargs):
        exists_dir = os.path.exists(args[0])
        if not exists_dir:
            os.makedirs(args[0])
        self._leveldb = leveldb.LevelDB(*args, **kwargs)

    def __getattr__(self, attr):
        return getattr(self._leveldb, attr)



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      mysql_batch_controller.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    zhouqinmin(zqm175899960@163.com)
    @date:      January 23, 2019
    @desc:      MysqlBatchController
"""

from tomato.util import singleton
from collections import namedtuple
from tomato.store.mysql.sql_clause_assemble import UnionType
from tomato.store.mysql.sql_clause_assemble import BatchOperator

MySqlBatchData = namedtuple('MySqlBatchData', ['Controller', 'Models', 'WhereParams', 'Method'])
BatchParams = namedtuple('BatchParams', ['sql_clause_param', 'batch_op'])

@singleton
class MysqlBatchController(object):
    __mysql_client = None
    union_strings = {UnionType.UNION: ' union ',
                    UnionType.UNION_ALL: ' union all '}

    def __init__(self, *args, **kwargs):
        pass

    @classmethod
    def set_mysql_client(cls, mysql_client):
        cls.__mysql_client = mysql_client

    async def insert_many_models(self, many_models, mysql_controller, where_params_list = None):
        data = MySqlBatchData(mysql_controller, many_models, where_params_list, BatchOperator.INSERT)
        result = await self.execute_batch([data])
        return result

    async def update_many_models(self, many_models, mysql_controller, where_params = None):
        data = MySqlBatchData(mysql_controller, many_models, where_params, BatchOperator.UPDATE)
        result = await self.execute_batch([data])
        return result

    async def execute_batch(self, mysql_batch_data_list):
        execute_batch_list = list()
        for data in mysql_batch_data_list:
            where_params = None
            if data.WhereParams is not None and isinstance(data.WhereParams, list):
                where_params = data.WhereParams
                execute_batch_list.extend([data.Controller.execute_sql_clause(data.Method, data.Models[i], where_params[i]) for i in range(len(data.Models))])
            else:
                where_params = data.WhereParams
                execute_batch_list.extend([data.Controller.execute_sql_clause(data.Method, model, where_params) for model in data.Models])
        result = await self.__mysql_client.execute_batch(execute_batch_list)
        return result

    async def execute_batch_params_list(self, batch_params_list):
        result = await self.__mysql_client.execute_batch([batch_param[0].execute_functions[batch_param[1]]() \
            for batch_param in batch_params_list])
        return result

    async def union(self, sql_clause_assemble_list, union_type):
        sql_list = list()
        sql_params = list()
        for sql_clause_assemble in sql_clause_assemble_list:
            (sql, params) = sql_clause_assemble.get_query_clause()
            sql_list.append(sql)
            sql_params.extend(params)
        result = await self.__mysql_client.get_all(self.union_strings[union_type].join(sql_list), sql_params)
        result_data = list()
        if result is not None and isinstance(result, list):
            result_data = [data for data in result]
        return result_data



#!/usr/bin/python
# -*- coding:utf-8 -*-

import os
import codecs
import chardet
import importlib
import configparser
import tornado.autoreload

def autoreload():
    tornado.autoreload.start()

def loadconf(conf_file, raw=False):
    filename = os.path.abspath(conf_file)
    if raw:
        conf = configparser.RawConfigParser()
    else:
        conf = configparser.ConfigParser()
    conf.readfp(codecs.open(filename, "r", "utf8"))
    return conf

def gen_object(module_name, class_name):
    """Generates an object based on module_name(xxx.xxx.xxx) and class_name
    """
    module = importlib.import_module(module_name)
    module_cls = getattr(module, class_name)
    cls_obj = module_cls()
    return cls_obj

def transcoding(text, encoding):
    """transcoding
    """
    textLength = 2048
    if len(text) < 2048:
        textLength = len(text)
    tempText = text[0:textLength]
    decoding = chardet.detect(tempText)['encoding']
    if decoding is None:
        text = text.decode('gbk', 'ignore').encode(encoding)
    elif not decoding is 'UTF-8':
        text = text.decode(decoding, 'ignore').encode(encoding)
    return text

from .log import Log
from .typeassert import typeassert
from .singleton import singleton
from .appmodule import AppModule
from .application import Application



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      redis_client.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    tangmi(tangmi360@gmail.com)
    @date:      June 11, 2018
    @desc:      Redis storage access class
"""

import aioredis


class RedisClient(aioredis.ConnectionsPool):

    def __init__(self, *args, **kwargs):
        super(RedisClient, self).__init__(*args, **kwargs)

    async def close(self):
        super().close()
        await self.wait_closed()



#!/usr/bin/python
# -*- coding:utf-8 -*-

import asyncio
import unittest

from tomato.util import Log
from tomato.transport import HttpClient
from tomato.transport import HttpServer


class TestHttpClient(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.log = Log(filename='unit_test.log', cmdlevel='INFO')
        cls.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(cls.loop)
        cls.http_server = HttpServer(host='localhost', port=8888)
        cls.loop.run_until_complete(cls.http_server.run())
        cls.http_client = HttpClient()

    @classmethod
    def tearDownClass(cls):
        cls.loop.run_until_complete(cls.http_client.close())
        cls.loop.run_until_complete(cls.http_server.close())
        cls.loop.run_until_complete(asyncio.sleep(0.250))
        cls.loop.stop()
        cls.loop.close()

    def setUp(self):
        self.url = 'http://localhost:8888'

    def tearDown(self):
        pass

    def test_http_status(self):
        self.log.info('test_http_status')
        async def http_status():
            http_get = self.http_client.get(self.url)
            response = await http_get
            return response.status

        status = self.loop.run_until_complete(http_status())
        self.assertEqual(status, 200)



#!/usr/bin/python
# -*- coding:utf-8 -*-

import os
import sys
from setuptools import setup, find_packages


if sys.version_info < (3, 5, 3):
    raise RuntimeError('aiohttp 3.x requires Python 3.5.3+')

with open('requirements.txt') as f:
    requirements = [l for l in f.read().splitlines() if l]

with open('README.rst', 'r') as fh:
    long_description = fh.read()

setup(
    name = 'tomatolib',
    version = '0.3.5',
    test_suite='nose.collector',
    tests_require=['nose'],
    packages = find_packages(),
    install_requires = requirements,
    author='tangmi',
    author_email='tangmi360@gmail.com',
    maintainer=', '.join(('tangmi <tangmi360@gmail.com>',
                          'stevenhu <love.as.string@gmail.com>')),
    maintainer_email='tomatolibs@gmail.com',
    description = 'A quick and lightweight server framework.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    data_files=[
        ('requires', ['requirements.txt']),
    ],
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: AsyncIO',
    ],
    python_requires='>=3.5.3',
    url='https://github.com/tangmi001/tomatolib',
    license="GNU General Public License v3",
    keywords = ['tomatolib', 'tomato']
)



#!/usr/bin/python
# -*- coding:utf-8 -*-

from abc import ABCMeta, abstractmethod


class AppModule(metaclass=ABCMeta):
    def __init__(self, *args, **kwargs):
        super(AppModule, self).__init__()

    def run(self):
        pass

    @abstractmethod
    def setup(self):
        pass

    @abstractmethod
    def destroy(self):
        pass



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
    @file:      sql_clause_assemble.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @auther:    zhouqinmin(zqm175899960@163.com)
    @date:      January 23, 2019
    @desc:      SqlClauseAssemble
"""

from enum import Enum
from collections import namedtuple
from tomato.store.mysql.sql_param_collections import SelectClauseParam
from tomato.store.mysql.sql_param_collections import SqlParamCollections


class TableJoin(Enum):
    INNER_JOIN = 1
    LEFT_JOIN = 2
    RIGHT_JOIN = 3

class BatchOperator(Enum):
    INSERT = 1
    DELETE = 2
    UPDATE = 3
    SELECT = 4

class UnionType(Enum):
    UNION = 1
    UNION_ALL = 2

JoinParams = namedtuple('JoinParams', ['JoinTable', 'JoinType', 'JoinWhereParams'])


class SqlClauseAssemble(object):
    join_strings = {TableJoin.INNER_JOIN: 'inner join',
                    TableJoin.LEFT_JOIN: 'left join',
                    TableJoin.RIGHT_JOIN: 'right join'}
    def __init__(self, 
        wanted_words=[], 
        insert_words=[], 
        update_words=[], 
        join_param_list=[], 
        table_name=None, 
        where_params=None):
        self.__where_params = where_params #SqlParamCollections
        self.__wanted_words = wanted_words #[keys]
        self.__insert_words = insert_words #[(k, v)]
        self.__update_words = update_words #[(k, v)]
        self.__table_name = table_name #str
        self.__join_param_list = join_param_list
        self.__execute_functions = {
            BatchOperator.SELECT: self.get_query_clause,
            BatchOperator.INSERT: self.get_insert_clause,
            BatchOperator.UPDATE: self.get_update_clause,
            BatchOperator.DELETE: self.get_delete_clause
        }

    @property
    def execute_functions(self):
        return self.__execute_functions

    @property
    def join_param_list(self):
        return self.__join_param_list

    @join_param_list.setter
    def join_param_list(self, value):
        """where_params
        """
        self.__join_param_list = value

    @property
    def where_params(self):
        return self.__where_params

    @where_params.setter
    def where_params(self, value):
        """where_params
        """
        self.__where_params = value

    @property
    def update_words(self):
        return self.__update_words

    @update_words.setter
    def update_words(self, value):
        """__update_words
        """
        self.__update_words = value

    @property
    def insert_words(self):
        return self.__insert_words

    @insert_words.setter
    def insert_words(self, value):
        """insert_words
        """
        self.__insert_words = value

    @property
    def wanted_words(self):
        return self.__wanted_words

    @wanted_words.setter
    def wanted_words(self, value):
        """wanted_words
        """
        self.__wanted_words = value

    @property
    def table_name(self):
        return self.__table_name

    @table_name.setter
    def table_name(self, value):
        """wanted_words
        """
        self.__table_name = value

    def __make_insert_words_sql_clause(self):
        keys = [kv[0] for kv in self.__insert_words]
        values = [kv[1] for kv in self.__insert_words]
        return (', '.join(keys), values)

    def __make_update_words_sql_clause(self):
        keys = ['%s = %%s' % kv[0] for kv in self.__update_words]
        values = [kv[1] for kv in self.__update_words]
        return (', '.join(keys), values)

    def __make_wanted_words_sql_clause(self):
        return ', '.join(self.__wanted_words) if self.__wanted_words and len(self.__wanted_words) != 0 else '*'

    def __check_and_extract_noraml_param(self, where_params):
        normal_param = where_params.data['normal_param']
        for i in range(len(normal_param)):
            p = normal_param[i]
            if isinstance(p[2], SqlClauseAssemble):
                (sql, params) = p[2].get_query_clause()
                normal_param[i] = (p[0], p[1], SelectClauseParam(sql, params))

    def get_query_clause(self):
        result_list = []
        result_params = []
        result_list.append('select')
        result_list.append(self.__make_wanted_words_sql_clause())
        result_list.append('from')
        result_list.append(self.__table_name)

        if len(self.join_param_list) != 0:
            for join_param in self.join_param_list:
                result_list.append(self.join_strings[join_param[1]])
                result_list.append(join_param[0])
                if join_param[2] is not None:
                    result_list.append('on')
                    (join_sql, join_params) = join_param[2].get_where_sql()
                    result_list.append(join_sql)
                    result_params.extend(join_params)

        if self.__where_params is not None:
            if self.__where_params.can_add_where():
                result_list.append('where')
                self.__check_and_extract_noraml_param(self.__where_params)
            (where_sql, params) = self.__where_params.get_where_sql()
            result_list.append(where_sql)
            result_params.extend(params)
        return (' '.join(result_list), result_params)

    def get_insert_clause(self):
        result_list = []
        result_params = []
        result_list.append('insert into')
        result_list.append(self.__table_name)
        result_list.append('(')
        (insert_words, insert_values) = self.__make_insert_words_sql_clause()
        result_list.append(insert_words)
        result_params.extend(insert_values)
        result_list.append(')')
        result_list.append('values(')
        result_list.append(', '.join(['%s'] * len(result_params)))
        result_list.append(')')
        if self.__where_params is not None and self.__where_params.can_add_where():
            result_list.append('ON DUPLICATE KEY UPDATE')
            if len(self.__where_params.data['linker_op_list']) == 0:
                self.__where_params.data['linker_op_list'].extend(len(self.__where_params.data['normal_param']) * [', '])
            (where_sql, params) = self.__where_params.get_where_sql()
            result_list.append(where_sql)
            result_params.extend(params)
        return (' '.join(result_list), result_params)

    def get_update_clause(self):
        result_list = []
        result_params = []
        result_list.append('update')
        result_list.append(self.__table_name)
        result_list.append('set')
        (update_words, update_values) = self.__make_update_words_sql_clause()
        result_list.append(update_words)
        result_params.extend(update_values)
        if self.__where_params is not None:
            if self.__where_params.can_add_where():
                result_list.append('where')
                self.__check_and_extract_noraml_param(self.__where_params)
            (where_sql, params) = self.__where_params.get_where_sql()
            result_list.append(where_sql)
            result_params.extend(params)
        return (' '.join(result_list), result_params)

    def get_delete_clause(self):
        result_list = []
        result_params = []
        result_list.append('delete from')
        result_list.append(self.__table_name)
        if self.__where_params is not None:
            if self.__where_params.can_add_where():
                result_list.append('where')
                self.__check_and_extract_noraml_param(self.__where_params)
            (where_sql, params) = self.__where_params.get_where_sql()
            result_list.append(where_sql)
            result_params.extend(params)
        return (' '.join(result_list), result_params)

def make_query_clause():
    id_sql_assemble = SqlClauseAssemble()
    id_sql_assemble.wanted_words = ['id', 'min(id)']
    id_sql_assemble.table_name = 'Persons'
    id_sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('age', '<', 35, True),
            ('Year', 'in', (2012, 2013, 2015), True)
            ]
    )

    sql_assemble = SqlClauseAssemble()
    sql_assemble.wanted_words = ['name']
    sql_assemble.table_name = 'Persons'
    sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('name', '=', 'zhouqinmin', True),
            ('id', 'in', id_sql_assemble, True)
            ]
    )

    total_sql_assemble = SqlClauseAssemble()
    total_sql_assemble.wanted_words = ['name']
    total_sql_assemble.table_name = 'Persons'
    total_sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('Year', '>', 1965, True),
            ('name', 'in', sql_assemble, True)
            ],
        order_by_param_list = [('age'), ('name', 'desc')],
        group_by_param_list=[('age'), ('Year')]
    )

    print(total_sql_assemble.get_query_clause())

def make_insert_clause():
    sql_assemble = SqlClauseAssemble()
    sql_assemble.insert_words = [('LastName', 'Gates'), ('FirstName', 'Bill'), 
        ('Address', 'Xuanwumen 10'), ('City', 'Beijing')]
    sql_assemble.table_name = 'Persons'
    print(sql_assemble.get_insert_clause())

def make_update_clause():
    sql_assemble_1 = SqlClauseAssemble()
    sql_assemble_1.wanted_words = ['name']
    sql_assemble_1.table_name = 'Persons'
    sql_assemble_1.where_params = SqlParamCollections(
        normal_param_list=[
            ('id', '=', '1', True),
            ]
    )

    id_sql_assemble = SqlClauseAssemble()
    id_sql_assemble.wanted_words = ['id']
    id_sql_assemble.table_name = 'Persons'
    id_sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('Year', '>', 1965, True),
            ('name', 'in', sql_assemble_1, True)
            ]
    )

    sql_assemble = SqlClauseAssemble()
    sql_assemble.update_words = [('Address', 'Zhongshan 23'), ('City', 'Nanjing')]
    sql_assemble.table_name = 'Person'
    sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('LastName', '=', 'Wilson', True),
            ('id', 'not in', id_sql_assemble, True)
            ]
    )
    print(sql_assemble.get_update_clause())

def make_delete_clause():
    id_sql_assemble = SqlClauseAssemble()
    id_sql_assemble.wanted_words = ['id']
    id_sql_assemble.table_name = 'Persons'
    id_sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('Year', '>', 1965, True)
            ]
    )

    sql_assemble = SqlClauseAssemble()
    sql_assemble.table_name = 'Person'
    sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('id', 'in', id_sql_assemble, True),
            ('LastName', '=', 'Wilson', True)
            ]
    )
    print(sql_assemble.get_delete_clause())

def make_join_query_clause():
    sql_assemble = SqlClauseAssemble()
    sql_assemble.wanted_words = ['name']
    sql_assemble.table_name = 'Persons'
    sql_assemble.where_params = SqlParamCollections(
        normal_param_list=[
            ('name', '=', 'zhouqinmin', True)
        ]
    )

    sql_assemble.join_param_list = \
    [('Student', TableJoin.LEFT_JOIN, SqlParamCollections(normal_param_list=[('Persons.id', '=', 'Student.id', False)])),
    ('Student', TableJoin.LEFT_JOIN, SqlParamCollections(normal_param_list=[('Persons.id', '=', 'Student.id', False)]))]

    print(sql_assemble.get_query_clause())

def main():
    make_query_clause()
    make_insert_clause()
    make_update_clause()
    make_delete_clause()
    make_join_query_clause()

if __name__ == '__main__':
    main()
